/*******************************************************************************
 *******************************************************************************
 **                                                                           **
 **                                                                           **
 **  Copyright 2003-2020 Tom Bascom, Greenfield Technologies                  **
 **  http://www.greenfieldtech.com                                            **
 **                                                                           **
 **  ProTop is free software; you can redistribute it and/or modify it        **
 **  under the terms of the GNU General Public License (GPL) as published     **
 **  by the Free Software Foundation; either version 2 of the License, or     **
 **  at your option) any later version.                                       **
 **                                                                           **
 **  ProTop is distributed in the hope that it will be useful, but WITHOUT    **
 **  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    **
 **  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License     **
 **  for more details.                                                        **
 **                                                                           **
 **  See TERMS.TXT for more information regarding the Terms and Conditions    **
 **  of use and alternative licensing options for this software.              **
 **                                                                           **
 **  A copy of the GPL is in GPL.TXT which was provided with this package.    **
 **                                                                           **
 **  See http://www.fsf.org for more information about the GPL.               **
 **                                                                           **
 **                                                                           **
 *******************************************************************************
 *******************************************************************************
 *
 * osinfo.p
 *
 */

{lib/protop.i}
{lib/protoplib.i}

define output parameter dcDescription as character no-undo initial "OSInfo".

function getNum returns decimal ( s as character ):

  s = trim( s ).

  if       substring( s, length( s ), 1 ) = "T" then return decimal( substring( s, 1, length( s ) - 1 )) * 1024 * 1024 * 1024 * 1024.
   else if substring( s, length( s ), 1 ) = "G" then return decimal( substring( s, 1, length( s ) - 1 )) * 1024 * 1024 * 1024.
   else if substring( s, length( s ), 1 ) = "M" then return decimal( substring( s, 1, length( s ) - 1 )) * 1024 * 1024.
   else if substring( s, length( s ), 1 ) = "K" then return decimal( substring( s, 1, length( s ) - 1 )) * 1024.
   else if substring( s, length( s ), 1 ) = "B" then return decimal( substring( s, 1, length( s ) - 1 )).
   else return decimal( s ).

end.

{lib/osinfo.i}

define variable pt_OSInfoFreqInt as integer no-undo.
define variable pt_OSInfoFreqBat as integer no-undo.

procedure mon-init:

  define variable osFamily as character no-undo.

  find tt_OSInfo no-error.
  if not available( tt_osinfo ) then create tt_osinfo.

  /* run osinfo. */

  input stream inStrm through value( "uname" ).
  import stream inStrm osFamily.
  input stream inStrm close.

  if osFamily <> "Linux" then	/* Linux is lightweight, we don't need to avoid running osInfo	*/
    do:

      pt_OSInfoFreqInt = integer( os-getenv( "OSINFOFREQINT" )) no-error.
      if pt_OSInfoFreqInt = ? then pt_OSInfoFreqInt = 60.

      pt_OSInfoFreqBat = integer( os-getenv( "OSINFOFREQBAT" )) no-error.
      if pt_OSInfoFreqBat = ? then pt_OSInfoFreqBat = 3600.

    end.

  return.

end.


define variable lastRefresh as datetime no-undo initial ?.

procedure mon-update:

  define input parameter argList as character no-undo.

  define variable inLine as character no-undo extent 16.
  define variable xLine  as character no-undo.

  find tt_OSInfo no-error.
  if not available( tt_osinfo ) then create tt_osinfo.

  /* other than uptime osinfo doesn't change much -- so don't update it with *every* sample
   *
   * this logic needs to move to lib/osinfo.i & the osinfo IP which should be called here
   *
   */

  if ( lastRefresh = ? or ( abs( interval( now, lastRefresh, "seconds" )) >= ( if session:batch = yes then pt_OSInfoFreqBat else pt_OSInfoFreqInt ))) then
    do:
      lastRefresh = now.
      run osinfo.
    end.

  if opsys begins "WIN" then
    do:
      osUpTime =
        string( abs ( interval( winBootDTZ, now, "days" )), ">>>>>>" ) + " " +
        string( abs ( interval( winBootDTZ, now, "seconds" )), "hh:mm:ss" )
      no-error.
      osUpTime = trim( osUpTime ).
    end.
   else
    do:
      input stream inStrm through value( "uptime" ).		/* for Linux we can also read /proc/uptime and avoid the shell	*/
      import stream inStrm unformatted xLine.
      input stream inStrm close.
      osUpTime = entry( 1, substring( xLine, index( xLine, "up" ) + 3 )).
    end.

  add2ds( temp-table tt_osinfo:default-buffer-handle ).

  return.

end.

return.
